"""MyGekko Blinds implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class Blind(Entity):
    """Class for MyGekko Blind"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: BlindValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/blinds/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[BlindFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def position(self) -> float | None:
        """Returns the current position"""
        value = self._value_accessor.get_value(self, "positionLevel")
        return float(value) if value is not None else None

    async def set_position(self, position: float):
        """Sets the position"""
        await self._value_accessor.set_position(self, position)

    @property
    def state(self) -> BlindState | None:
        """Returns the current state"""
        value = self._value_accessor.get_value(self, "currentState")
        return BlindState(int(value)) if value is not None else None

    async def set_state(self, state: BlindState):
        """Sets the state"""
        await self._value_accessor.set_state(self, state)

    @property
    def tilt_position(self) -> float | None:
        """Returns the current tilt position"""
        value = self._value_accessor.get_value(self, "rotationLevel")
        return float(value) if value is not None else None

    async def set_tilt_position(self, position: float):
        """Sets the tilt position"""
        await self._value_accessor.set_tilt_position(self, position)

    @property
    def element_info(self) -> BlindElementInfo | None:
        """Returns the element info"""
        value = self._value_accessor.get_value(self, "elementInfo")
        return BlindElementInfo(float(value)) if value is not None else None


class BlindState(IntEnum):
    """MyGekko Blinds State"""

    HOLD_DOWN = -2
    DOWN = -1
    STOP = 0
    UP = 1
    HOLD_UP = 2


class BlindFeature(IntEnum):
    """MyGekko Blinds Feature"""

    OPEN_CLOSE_STOP = 0
    SET_POSITION = 1
    SET_TILT_POSITION = 2
    OPEN_CLOSE = 3


class BlindElementInfo(IntEnum):
    """MyGekko Blinds Element Info"""

    OK = 0
    MANUAL_OFF = 1
    MANUAL_ON = 2
    LOCKED = 3
    ALARM = 4


class BlindValueAccessor(EntityValueAccessor):
    """Blind value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        self._data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "blinds" in status:
            blinds = status["blinds"]
            for key in blinds:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in blinds[key] and "value" in blinds[key]["sumstate"]:
                        (
                            self._data[key]["currentState"],
                            self._data[key]["positionLevel"],
                            self._data[key]["rotationLevel"],
                            self._data[key]["elementInfo"],
                            self._data[key]["rotationRange"],
                            *_other,
                        ) = blinds[key]["sumstate"]["value"].split(
                            ";",
                        )

                if key.startswith("group"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in blinds[key] and "value" in blinds[key]["sumstate"]:
                        (
                            self._data[key]["currentState"],
                            *_other,
                        ) = blinds[key][
                            "sumstate"
                        ]["value"].split(
                            ";",
                        )

    def update_resources(self, resources):
        if resources is not None and "blinds" in resources:
            blinds = resources["blinds"]
            for key in blinds:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = blinds[key]["name"]

                if key.startswith("group"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = blinds[key]["name"]

    @property
    def blinds(self):
        """Returns the blinds read from MyGekko"""
        result: list[Blind] = []
        for key, data in self._data.items():
            result.append(Blind(key, data["name"], self))

        return result

    def get_features(self, blind: Blind) -> list[BlindFeature]:
        """Returns the supported features"""
        result = list()

        if blind and blind.entity_id:
            if blind.entity_id in self._data:
                data = self._data[blind.entity_id]
                if blind.entity_id.startswith("group"):
                    # open/close feature is the only feature for groups
                    result.append(BlindFeature.OPEN_CLOSE)
                    return result

                if "currentState" in data and data["currentState"]:
                    result.append(BlindFeature.OPEN_CLOSE_STOP)

                if "positionLevel" in data and data["positionLevel"]:
                    result.append(BlindFeature.SET_POSITION)

                if "rotationLevel" in data and data["rotationLevel"]:
                    result.append(BlindFeature.SET_TILT_POSITION)

        return result

    async def set_position(self, blind: Blind, position: float) -> None:
        """Sets the position"""
        if blind and blind.entity_id and position >= 0 and position <= 100.0:
            await self._data_provider.write_data(
                blind.resource_path, "P" + str(position)
            )

    async def set_tilt_position(self, blind: Blind, position: float) -> None:
        """Sets the tilt position"""
        if blind and blind.entity_id and position >= 0 and position <= 100.0:
            await self._data_provider.write_data(
                blind.resource_path, "S" + str(position)
            )

    async def set_state(self, blind: Blind, state: BlindState) -> None:
        """Sets the state"""
        if blind and blind.entity_id:
            await self._data_provider.write_data(blind.resource_path, str(state))



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase


async def var_response(request):
    varResponseFile = open("tests/alarms_logics/data/api_var_response_596610.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open(
        "tests/alarms_logics/data/api_var_status_response_596610.json"
    )
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_alarms_logics(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        alarms_logics = api.get_alarms_logics()

        assert alarms_logics is not None
        assert len(alarms_logics) == 2

        assert alarms_logics[0].entity_id == "item0"
        assert alarms_logics[0].name == "Button"
        assert alarms_logics[0].value == 1.0

        assert alarms_logics[1].entity_id == "item15"
        assert alarms_logics[1].name == "Klingel"
        assert alarms_logics[1].value == 0.0



# SPDX-FileCopyrightText: 2023-present Stephan Uhle <stephanu@gmx.net>
#
# SPDX-License-Identifier: MIT


class ReadOnlyEntity:
    """Base class for read only MyGekko entities"""

    def __init__(self, entity_id: str, name: str) -> None:
        self.entity_id = entity_id
        self.name = name


class Entity(ReadOnlyEntity):
    """Base class for MyGekko entities"""

    def __init__(self, entity_id: str, name: str, resource_path_prefix: str) -> None:
        super().__init__(entity_id, name)
        self._resource_path = resource_path_prefix + self.entity_id

    @property
    def resource_path(self) -> str:
        """Returns the resource path of this entity"""
        return self._resource_path



"""Base implementation of the data provider"""
import json
import logging
import pkgutil
from abc import ABC
from abc import abstractmethod

from aiohttp import ClientSession
from yarl import URL

from .resources import Entity

_LOGGER: logging.Logger = logging.getLogger(__name__)


class DataSubscriberInterface:
    """Interface for data subscribers"""

    def update_status(self, status, hardware):
        """Method called on status updates"""

    def update_resources(self, resources):
        """Method called on resources updates"""


class EntityValueAccessor(DataSubscriberInterface):
    """Base class for entity values accessors"""

    _data = {}

    def get_value(self, entity: Entity, value_name: str) -> str | None:
        """Returns a data value of this entity"""
        if entity and entity.entity_id:
            if (
                entity.entity_id in self._data
                and value_name in self._data[entity.entity_id]
                and self._data[entity.entity_id][value_name]
            ):
                return self._data[entity.entity_id][value_name]
        return None


class DataProviderBase(ABC):
    """Base class for data providers"""

    _subscriber: list[DataSubscriberInterface] = []
    _status = None
    _resources = None

    @property
    def resources(self):
        """returns the read resources"""
        return self._resources

    @resources.setter
    def resources(self, resources):
        self._resources = resources
        for subscriber in self._subscriber:
            subscriber.update_resources(self._resources)

    @property
    def status(self):
        """returns the read status"""
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

        # current assumption is, that the hardware property is not set in legacy hardware (Slide)
        # but only in newer hardware (Slide 2, Nova)
        hardware = "legacy"
        network_data = status["globals"]["network"]
        if (
            "hardware" in network_data
            and network_data["hardware"]
            and "value" in network_data["hardware"]
        ):
            hardware = network_data["hardware"]["value"]

        for subscriber in self._subscriber:
            subscriber.update_status(self._status, hardware)

    def subscribe(self, subscriber: DataSubscriberInterface):
        """Method to subscribe to data changes."""
        self._subscriber.append(subscriber)

    @abstractmethod
    async def read_data(self) -> None:
        """Reads data from the MyGekko API"""

    @abstractmethod
    async def write_data(self, resource_path: str, value: str):
        """Sends data to the MyGekko API"""

    @abstractmethod
    async def try_connect(self) -> None:
        """Tries to connect to the MyGekko API using the given credentials"""


class DummyDataProvider(DataProviderBase):
    """Dummy data provider returning static test data"""

    async def try_connect(self) -> None:
        _LOGGER.debug("try_connect in DummyDataProvider")

    async def read_data(self) -> None:
        _LOGGER.debug("read_data in DummyDataProvider")
        var_demo_data = pkgutil.get_data(__name__, "api_var_demo_data.json")
        self.resources = json.loads(var_demo_data)

        status_demo_data = pkgutil.get_data(__name__, "api_var_status_demo_data.json")
        self.status = json.loads(status_demo_data)

    async def write_data(self, resource_path: str, value: str):
        _LOGGER.info("Writing to %s %s", resource_path, value)


class DataProvider(DataProviderBase):
    """Data provider accessing the MyGekko API"""

    def __init__(
        self, url: URL, authentication_params: dict[str, str], session: ClientSession
    ):
        self._url = url
        self._authentication_params = authentication_params
        self._session = session

    async def try_connect(self) -> None:
        _LOGGER.debug("try_connect in DataProvider")

        async with self._session.get(
            self._url.with_path("/api/v1/var"), params=self._authentication_params
        ) as resp:
            response_text = await resp.text()
            if resp.status == 200:
                _LOGGER.debug("Ok %s", response_text)
            else:
                self.handle_api_error(resp.status, response_text)

    async def read_data(self) -> None:
        _LOGGER.debug("read_data in DataProvider: /api/v1/var")
        async with self._session.get(
            self._url.with_path("/api/v1/var"),
            params=self._authentication_params,
        ) as resp:
            response_text = await resp.text()
            if resp.status == 200:
                try:
                    self.resources = json.loads(response_text)
                except json.JSONDecodeError:
                    _LOGGER.exception("Json Parsing the response failed")
            else:
                _LOGGER.error(
                    "Error reading the resources %s %s", resp.status, await resp.text()
                )
                self.handle_api_error(resp.status, response_text)

        _LOGGER.debug("read_data in DataProvider: /api/v1/var/status")
        async with self._session.get(
            self._url.with_path("/api/v1/var/status"),
            params=self._authentication_params,
        ) as resp:
            response_text = await resp.text()
            if resp.status == 200:
                try:
                    self.status = json.loads(response_text)
                except json.JSONDecodeError:
                    _LOGGER.exception("Json Parsing the response failed")
            else:
                _LOGGER.error(
                    "Error reading the resources %s %s", resp.status, await resp.text()
                )
                self.handle_api_error(resp.status, response_text)

        _LOGGER.debug("read_data end")

    async def write_data(self, resource_path: str, value: str):
        resource_url = "/api/v1/var" + resource_path + "/scmd/set"

        _LOGGER.debug("Writing data %s to %s", value, resource_url)

        async with self._session.get(
            self._url.with_path(resource_url),
            params=self._authentication_params | {"value": value},
        ) as resp:
            if resp.status != 200:
                response_text = await resp.text()
                self.handle_api_error(resp.status, response_text)

    def handle_api_error(self, response_status, response_text):
        """Handles the api response in case of errors"""
        if response_status == 400:
            _LOGGER.error("MyGekkoBadRequest %s", response_text)
            raise MyGekkoBadRequest()
        elif response_status == 403:
            _LOGGER.error("MyGekkoForbidden %s", response_text)
            raise MyGekkoForbidden()
        elif response_status == 404:
            _LOGGER.error("MyGekkoNotFound %s", response_text)
            raise MyGekkoNotFound()
        elif response_status == 405:
            _LOGGER.error("MyGekkoMethodNotAllowed %s", response_text)
            raise MyGekkoMethodNotAllowed()
        elif response_status == 410:
            _LOGGER.error("MyGekkoGone %s", response_text)
            raise MyGekkoGone()
        elif response_status == 429:
            _LOGGER.error("MyGekkoTooManyRequests %s", response_text)
            raise MyGekkoTooManyRequests()
        elif response_status == 444:
            _LOGGER.error("MyGekkoNoResponse %s", response_text)
            raise MyGekkoNoResponse()
        else:
            raise MyGekkoError()


class MyGekkoError(Exception):
    """Base MyGekko exception."""


class MyGekkoBadRequest(MyGekkoError):
    """MyGekko Bad Request exception."""


class MyGekkoForbidden(MyGekkoError):
    """MyGekko Forbidden exception."""


class MyGekkoNotFound(MyGekkoError):
    """MyGekko Not Found exception."""


class MyGekkoMethodNotAllowed(MyGekkoError):
    """MyGekko Method Not Allowed exception."""


class MyGekkoGone(MyGekkoError):
    """MyGekko Gone exception."""


class MyGekkoTooManyRequests(MyGekkoError):
    """MyGekko Too Many Requests exception."""


class MyGekkoNoResponse(MyGekkoError):
    """MyGekko No Response exception."""



"""MyGekko RoomTemps implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class RoomTemp(Entity):
    """Class for MyGekko RoomTemp"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: RoomTempsValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/roomtemps/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[RoomTempsFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def working_mode(self) -> RoomTempsMode | None:
        """ "Returns the current working mode"""
        value = self._value_accessor.get_value(self, "workingMode")
        return RoomTempsMode(int(value)) if value is not None else None

    async def set_working_mode(self, working_mode: RoomTempsMode):
        """Sets the working mode"""
        await self._value_accessor.set_working_mode(self, working_mode)

    @property
    def current_temperature(self) -> float | None:
        """Returns the current  temperature"""
        value = self._value_accessor.get_value(self, "temperatureValue")
        return float(value) if value is not None else None

    @property
    def target_temperature(self) -> float | None:
        """Returns the current target temperature"""
        value = self._value_accessor.get_value(self, "temperatureSetPointValue")
        return float(value) if value is not None else None

    async def set_target_temperature(self, target_temperature: float):
        """Sets the target temperature"""
        await self._value_accessor.set_target_temperature(self, target_temperature)

    @property
    def humidity(self) -> float | None:
        """Returns the current humidity"""
        value = self._value_accessor.get_value(self, "relativeHumidityLevel")
        return float(value) if value is not None else None

    @property
    def air_quality(self) -> float | None:
        """Returns the current air quality"""
        value = self._value_accessor.get_value(self, "airQualityLevel")
        return float(value) if value is not None else None


class RoomTempsMode(IntEnum):
    """MyGekko RoomTemps Mode"""

    OFF = 1
    COMFORT = 8
    REDUCED = 16
    MANUAL = 64
    STANDBY = 256


class RoomTempsFeature(IntEnum):
    """MyGekko RoomTemps Feature"""

    TARGET_TEMPERATURE = 0
    AIR_QUALITY = 1
    HUMIDITY = 2


class RoomTempsValueAccessor(EntityValueAccessor):
    """RoomTemps value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        self._data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "roomtemps" in status:
            room_temps = status["roomtemps"]
            for key in room_temps:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if (
                        "sumstate" in room_temps[key]
                        and "value" in room_temps[key]["sumstate"]
                    ):
                        self.read_value(
                            key, room_temps[key]["sumstate"]["value"], hardware
                        )

    def read_value(self, key, value, hardware):
        """Reads a value based on the given hardware."""
        if hardware == "legacy":
            (
                self._data[key]["temperatureValue"],
                self._data[key]["temperatureSetPointValue"],
                self._data[key]["valveOpeningLevel"],
                self._data[key]["workingMode"],
                self._data[key]["Reserved"],
                self._data[key]["temperatureAdjustmentValue"],
                self._data[key]["coolingModeState"],
                self._data[key]["elementInfo"],
                *_other,
            ) = value.split(";")
        else:
            (
                self._data[key]["temperatureValue"],
                self._data[key]["temperatureSetPointValue"],
                self._data[key]["valveOpeningLevel"],
                self._data[key]["workingMode"],
                self._data[key]["Reserved"],
                self._data[key]["temperatureAdjustmentValue"],
                self._data[key]["coolingModeState"],
                self._data[key]["elementInfo"],
                self._data[key]["relativeHumidityLevel"],
                self._data[key]["airQualityLevel"],
                self._data[key]["floorTemperatureValue"],
                *_other,
            ) = value.split(";")

    def update_resources(self, resources):
        if resources is not None and "roomtemps" in resources:
            room_temps = resources["roomtemps"]
            for key in room_temps:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = room_temps[key]["name"]

    @property
    def room_temps(self):
        """Returns the loads read from MyGekko"""
        result: list[RoomTemp] = []
        for key, data in self._data.items():
            result.append(RoomTemp(key, data["name"], self))

        return result

    def get_features(self, room_temp: RoomTemp) -> list[RoomTempsFeature]:
        """Returns the supported features"""
        result = list()

        if room_temp and room_temp.entity_id:
            if room_temp.entity_id in self._data:
                data = self._data[room_temp.entity_id]
                if (
                    "temperatureSetPointValue" in data
                    and data["temperatureSetPointValue"]
                ):
                    result.append(RoomTempsFeature.TARGET_TEMPERATURE)

                if "relativeHumidityLevel" in data and data["relativeHumidityLevel"]:
                    result.append(RoomTempsFeature.HUMIDITY)

                if "airQualityLevel" in data and data["airQualityLevel"]:
                    result.append(RoomTempsFeature.AIR_QUALITY)

        return result

    async def set_target_temperature(
        self, room_temp: RoomTemp, target_temperature: float
    ) -> None:
        """Sets the target temperature"""
        if room_temp and room_temp.entity_id:
            await self._data_provider.write_data(
                room_temp.resource_path, "S" + str(target_temperature)
            )

    async def set_working_mode(
        self, room_temp: RoomTemp, working_mode: RoomTempsMode
    ) -> None:
        """Sets the working mode"""
        if room_temp and room_temp.entity_id:
            await self._data_provider.write_data(
                room_temp.resource_path, "M" + str(working_mode)
            )



# SPDX-FileCopyrightText: 2023-present Stephan Uhle <stephanu@gmx.net>
#
# SPDX-License-Identifier: MIT
import logging
from typing import Union
from urllib import parse

from aiohttp import ClientSession
from PyMyGekko.resources.Actions import Action
from PyMyGekko.resources.Actions import ActionValueAccessor
from PyMyGekko.resources.AlarmsLogics import AlarmsLogic
from PyMyGekko.resources.AlarmsLogics import AlarmsLogicValueAccessor
from PyMyGekko.resources.Blinds import Blind
from PyMyGekko.resources.Blinds import BlindValueAccessor
from PyMyGekko.resources.EnergyCosts import EnergyCost
from PyMyGekko.resources.EnergyCosts import EnergyCostValueAccessor
from PyMyGekko.resources.HotWaterSystems import HotWaterSystem
from PyMyGekko.resources.HotWaterSystems import HotWaterSystemValueAccessor
from PyMyGekko.resources.Lights import Light
from PyMyGekko.resources.Lights import LightValueAccessor
from PyMyGekko.resources.Loads import Load
from PyMyGekko.resources.Loads import LoadValueAccessor
from PyMyGekko.resources.Meteo import Meteo
from PyMyGekko.resources.Meteo import MeteoValueAccessor
from PyMyGekko.resources.RoomTemps import RoomTemp
from PyMyGekko.resources.RoomTemps import RoomTempsValueAccessor
from PyMyGekko.resources.vents import Vent
from PyMyGekko.resources.vents import VentValueAccessor
from yarl import URL

from .data_provider import DataProvider
from .data_provider import DummyDataProvider


_LOGGER: logging.Logger = logging.getLogger(__name__)


class MyGekkoApiClientBase:
    """The base class for the MyGekko api client"""

    def __init__(
        self,
        authentication_params: dict[str, str] = None,
        session: ClientSession = None,
        demo_mode: bool = False,
        scheme: str = "https",
        host: str = "live.my-gekko.com",
        port: Union[int, None] = None,
    ) -> None:
        self._url = URL.build(scheme=scheme, host=host, port=port)
        self._authentication_params = authentication_params
        self._session = session
        self._demo_mode = demo_mode

        _LOGGER.debug(
            "Initializing MyGekkoApiClientBase demo_mode: %s", self._demo_mode
        )

        if self._demo_mode:
            self._data_provider = DummyDataProvider()
        else:
            self._data_provider = DataProvider(
                self._url, self._authentication_params, self._session
            )

        self._actions_value_accessor = ActionValueAccessor(self._data_provider)
        self._alarm_logics_value_accessor = AlarmsLogicValueAccessor(
            self._data_provider
        )
        self._blind_value_accessor = BlindValueAccessor(self._data_provider)
        self._energy_costs_value_accessor = EnergyCostValueAccessor(self._data_provider)
        self._hot_water_systems_value_accessor = HotWaterSystemValueAccessor(
            self._data_provider
        )
        self._light_value_accessor = LightValueAccessor(self._data_provider)
        self._loads_value_accessor = LoadValueAccessor(self._data_provider)
        self._room_temps_value_accessor = RoomTempsValueAccessor(self._data_provider)
        self._vents_value_accessor = VentValueAccessor(self._data_provider)
        self._meteo_value_accessor = MeteoValueAccessor(self._data_provider)

    async def try_connect(self) -> None:
        """Tries to connect to the MyGekko API using the given credentials"""
        await self._data_provider.try_connect()

    async def read_data(self) -> None:
        """Reads the status and resources data via the MyGekko API"""
        await self._data_provider.read_data()

    def get_globals_network(self):
        """Returns the globals network information"""
        if self._data_provider.status is None:
            return None

        result = {}
        if (
            self._data_provider.status["globals"]
            and self._data_provider.status["globals"]["network"]
        ):
            network_data = self._data_provider.status["globals"]["network"]
            for key in network_data:
                result[key] = network_data[key]["value"]

        return result

    def get_actions(self) -> list[Action]:
        """Returns the MyGekko actions"""
        return self._actions_value_accessor.actions

    def get_alarms_logics(self) -> list[AlarmsLogic]:
        """Returns the MyGekko alarms_logics"""
        return self._alarm_logics_value_accessor.alarms_logics

    def get_blinds(self) -> list[Blind]:
        """Returns the MyGekko blinds"""
        return self._blind_value_accessor.blinds

    def get_energy_costs(self) -> list[EnergyCost]:
        """Returns the MyGekko energy_costs"""
        return self._energy_costs_value_accessor.energy_costs

    def get_hot_water_systems(self) -> list[HotWaterSystem]:
        """Returns the MyGekko hot_water_systems"""
        return self._hot_water_systems_value_accessor.hotwater_systems

    def get_lights(self) -> list[Light]:
        """Returns the MyGekko lights"""
        return self._light_value_accessor.lights

    def get_loads(self) -> list[Load]:
        """Returns the MyGekko load"""
        return self._loads_value_accessor.loads

    def get_room_temps(self) -> list[RoomTemp]:
        """Returns the MyGekko room_temps"""
        return self._room_temps_value_accessor.room_temps

    def get_vents(self) -> list[Vent]:
        """Returns the MyGekko vents"""
        return self._vents_value_accessor.vents

    def get_meteo(self) -> Meteo:
        """Returns the MyGekko meteo"""
        return self._meteo_value_accessor.meteo


class MyGekkoQueryApiClient(MyGekkoApiClientBase):
    """The api client to access MyGekko via the MyGekko query api"""

    def __init__(
        self,
        username: str,
        api_key: str,
        gekko_id: str,
        session: ClientSession,
    ) -> None:
        super().__init__(
            authentication_params={
                "username": username,
                "key": api_key,
                "gekkoid": gekko_id,
            },
            session=session,
        )


class MyGekkoLocalApiClient(MyGekkoApiClientBase):
    """The api client to access MyGekko locally."""

    def __init__(
        self,
        username: str,
        password: str,
        session: ClientSession,
        host: str,
    ) -> None:
        super().__init__(
            authentication_params={
                "username": username,
                "password": parse.quote(password),
            },
            session=session,
            host=host,
        )


class MyGekkoDemoModeClient(MyGekkoApiClientBase):
    """The api client in demo mode."""

    def __init__(
        self,
    ) -> None:
        super().__init__(
            demo_mode=True,
        )



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.Actions import ActionFeature
from PyMyGekko.resources.Actions import ActionState


async def var_response(request):
    varResponseFile = open("tests/actions/data/api_var_response_596610.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open("tests/actions/data/api_var_status_response_596610.json")
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_actions(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        actions = api.get_actions()

        assert actions is not None
        assert len(actions) == 12

        assert actions[0].entity_id == "item0"
        assert actions[0].name == "Heizung AUS"
        assert actions[0].state == ActionState.OFF
        assert len(actions[0].supported_features) == 1
        assert ActionFeature.ON_OFF in actions[0].supported_features

        assert actions[1].entity_id == "item1"
        assert actions[1].name == "Heizung EIN"
        assert actions[1].state == ActionState.OFF
        assert len(actions[1].supported_features) == 1
        assert ActionFeature.ON_OFF in actions[1].supported_features

        assert actions[2].entity_id == "item2"
        assert actions[2].name == "Alle Rollos STOP"
        assert actions[2].state == ActionState.OFF
        assert len(actions[2].supported_features) == 1
        assert ActionFeature.ON_OFF in actions[2].supported_features



"""MyGekko Vents implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class Vent(Entity):
    """Class for MyGekko Vents"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: VentValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/vents/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[VentFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def element_info(self) -> VentElementInfo | None:
        """Returns the element info"""
        value = self._value_accessor.get_value(self, "elementInfo")
        return VentElementInfo(int(value)) if value is not None else None

    @property
    def working_level(self) -> VentWorkingLevel | None:
        """Returns the current working level"""
        value = self._value_accessor.get_value(self, "workingLevel")
        return VentWorkingLevel(int(value)) if value is not None else None

    @property
    def maximum_working_level(self) -> VentWorkingLevel | None:
        """Returns the maximum working level"""
        value = self._value_accessor.get_value(self, "maximumWorkingLevel")
        return VentWorkingLevel(int(value)) if value is not None else None

    async def set_working_level(self, working_level: VentWorkingLevel):
        """Sets the working level"""
        await self._value_accessor.set_working_level(self, working_level)

    @property
    def device_model(self) -> VentDeviceModel | None:
        """Returns the device_model"""
        value = self._value_accessor.get_value(self, "deviceModel")
        return VentDeviceModel(int(value)) if value is not None else None

    @property
    def relative_humidity(self) -> float | None:
        """Returns the relative humidity"""
        value = self._value_accessor.get_value(self, "relativeHumidityLevel")
        return float(value) if value is not None else None

    @property
    def air_quality(self) -> float | None:
        """Returns the air quality"""
        value = self._value_accessor.get_value(self, "airQualityLevel")
        return float(value) if value is not None else None

    @property
    def co2(self) -> float | None:
        """Returns the co2 level"""
        value = self._value_accessor.get_value(self, "co2Value")
        return float(value) if value is not None else None

    @property
    def supply_air_temperature(self) -> float | None:
        """Returns the supply air temperature"""
        value = self._value_accessor.get_value(self, "supplyAirTemperatureValue")
        return float(value) if value is not None else None

    @property
    def exhaust_air_temperature(self) -> float | None:
        """Returns the exhaust air temperature"""
        value = self._value_accessor.get_value(self, "exhaustAirTemperatureValue")
        return float(value) if value is not None else None

    @property
    def outside_air_temperature(self) -> float | None:
        """Returns the outside air temperature"""
        value = self._value_accessor.get_value(self, "outsideAirTemperatureValue")
        return float(value) if value is not None else None

    @property
    def outgoing_air_temperature(self) -> float | None:
        """Returns the outgoing air temperature"""
        value = self._value_accessor.get_value(self, "outgoingAirTemperatureValue")
        return float(value) if value is not None else None

    @property
    def supply_air_working_level(self) -> float | None:
        """Returns the supply air working level"""
        value = self._value_accessor.get_value(self, "supplyAirWorkingLevel")
        return float(value) if value is not None else None

    @property
    def exhaust_air_working_level(self) -> float | None:
        """Returns the exhaust air working level"""
        value = self._value_accessor.get_value(self, "exhaustAirWorkingLevel")
        return float(value) if value is not None else None

    @property
    def cooling_mode(self) -> VentCoolingMode | None:
        """Returns the cooling mode"""
        value = self._value_accessor.get_value(self, "coolingModeState")
        return VentCoolingMode(int(value)) if value is not None else None

    async def set_cooling_mode(self, cooling_mode: VentCoolingMode):
        """Sets the cooling mode"""
        await self._value_accessor.set_cooling_mode(self, cooling_mode)

    @property
    def dehumid_mode(self) -> VentDehumidMode | None:
        """Returns the dehumid mode"""
        value = self._value_accessor.get_value(self, "dehumidModeState")
        return VentDehumidMode(int(value)) if value is not None else None

    async def set_dehumid_mode(self, dehumid_mode: VentDehumidMode):
        """Sets the dehumid mode"""
        await self._value_accessor.set_dehumid_mode(self, dehumid_mode)

    @property
    def bypass_mode(self) -> VentBypassMode | None:
        """Returns the bypass mode"""
        value = self._value_accessor.get_value(self, "bypassMode")
        return VentBypassMode(int(value)) if value is not None else None

    @property
    def bypass_state(self) -> VentBypassState | None:
        """Returns the bypass state"""
        value = self._value_accessor.get_value(self, "bypassState")
        return VentBypassState(int(value)) if value is not None else None

    async def set_bypass_state(self, bypass_state: VentBypassState):
        """Sets the bypass state"""
        await self._value_accessor.set_bypass_state(self, bypass_state)

    @property
    def working_mode(self) -> VentWorkingMode | VentWorkingModeZimmermann | None:
        """Returns the working mode"""
        value = self._value_accessor.get_value(self, "workingMode")
        if self.device_model in [
            VentDeviceModel.ZIMMERMANN_V1,
            VentDeviceModel.ZIMMERMANN_V2,
        ]:
            return VentWorkingModeZimmermann(int(value)) if value is not None else None
        else:
            return VentWorkingMode(int(value)) if value is not None else None

    async def set_working_mode(
        self, working_mode: VentWorkingMode | VentWorkingModeZimmermann
    ):
        """Sets the working mode"""
        await self._value_accessor.set_working_mode(self, working_mode)

    @property
    def sub_working_mode(
        self,
    ) -> VentSubWorkingMode | VentSubWorkingModeZimmermann | None:
        """Returns the mode"""
        value = self._value_accessor.get_value(self, "subWorkingMode")
        if self.device_model in [
            VentDeviceModel.ZIMMERMANN_V1,
            VentDeviceModel.ZIMMERMANN_V2,
        ]:
            return (
                VentSubWorkingModeZimmermann(int(value)) if value is not None else None
            )
        else:
            return VentSubWorkingMode(int(value)) if value is not None else None


class VentWorkingMode(IntEnum):
    """MyGekko Vent Working Mode"""

    AUTO = 0
    MANUAL = 1
    PLUGGIT_AUTO = 2
    PLUGGIT_WEEK = 3


class VentWorkingModeZimmermann(IntEnum):
    """MyGekko Vent Working Mode for Zimmermann"""

    OFF = 0
    ECO_SUMMER = 1
    ECO_WINTER = 2
    COMFORT = 3
    STOVE = 4


class VentSubWorkingMode(IntEnum):
    """MyGekko Vent SubWorkingMode"""

    EXHAUST = 0
    EXHAUST_SUPPLY = 1
    EXHAUST_SUPPLY_HEAT_RECOVERY = 2


class VentSubWorkingModeZimmermann(IntEnum):
    """MyGekko Vent SubWorkingMode for Zimmermann"""

    MIDDLE_TEMP_OFFSET = 0
    ONLY_OFFSET = 1


class VentWorkingLevel(IntEnum):
    """MyGekko Vent Working Level"""

    OFF = 0
    LEVEL_1 = 1
    LEVEL_2 = 2
    LEVEL_3 = 3
    LEVEL_4 = 4


class VentDeviceModel(IntEnum):
    """MyGekko Vent Device Model"""

    STANDARD = 0
    PLUGGIT = 1
    ZIMMERMANN_V1 = 2
    WESTAFLEX = 3
    STIEBEL_LWZ = 4
    ZIMMERMANN_V2 = 5


class VentFeature(IntEnum):
    """MyGekko Vent Features"""

    WORKING_LEVEL = 0
    CO2 = 1
    HUMIDITY = 2
    AIR_QUALITY = 3


class VentElementInfo(IntEnum):
    """MyGekko Vent ElementInfo"""

    OK = 0
    MANUAL_OFF = 1
    MANUAL_ON = 2
    LOCKED = 3
    ALARM = 4


class VentCoolingMode(IntEnum):
    """MyGekko Vent Cooling Mode"""

    OFF = 0
    ON = 1


class VentDehumidMode(IntEnum):
    """MyGekko Vent Dehumid Mode"""

    OFF = 0
    ON = 1


class VentBypassMode(IntEnum):
    """MyGekko Vent Bypass Mode"""

    AUTO = 0
    MANUAL = 1
    SUMMER = 2


class VentBypassState(IntEnum):
    """MyGekko Vent Bypass State"""

    AUTO = 0
    MANUAL = 1
    SUMMER = 2


class VentValueAccessor(EntityValueAccessor):
    """Vent value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        self._data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "vents" in status:
            vents = status["vents"]
            for key in vents:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in vents[key] and "value" in vents[key]["sumstate"]:
                        self.read_value(key, vents[key]["sumstate"]["value"], hardware)

    def read_value(self, key, value, hardware):
        """Reads a value based on the given hardware."""
        if hardware == "legacy":
            (
                self._data[key]["workingLevel"],
                self._data[key]["deviceModel"],
                self._data[key]["workingMode"],
                self._data[key]["bypassState"],
                self._data[key]["maximumWorkingLevel"],
                self._data[key]["relativeHumidityLevel"],
                self._data[key]["co2Value"],
                self._data[key]["airQualityLevel"],
                self._data[key]["supplyAirTemperatureValue"],
                self._data[key]["exhaustAirTemperatureValue"],
                self._data[key]["outsideAirTemperatureValue"],
                self._data[key]["outgoingAirTemperatureValue"],
                self._data[key]["supplyAirWorkingLevel"],
                self._data[key]["exhaustAirWorkingLevel"],
                self._data[key]["elementInfo"],
                *_other,
            ) = value.split(
                ";",
            )
        else:
            (
                self._data[key]["workingLevel"],
                self._data[key]["deviceModel"],
                self._data[key]["workingMode"],
                self._data[key]["bypassState"],
                self._data[key]["maximumWorkingLevel"],
                self._data[key]["relativeHumidityLevel"],
                self._data[key]["airQualityLevel"],
                self._data[key]["co2Value"],
                self._data[key]["supplyAirTemperatureValue"],
                self._data[key]["exhaustAirTemperatureValue"],
                self._data[key]["outsideAirTemperatureValue"],
                self._data[key]["outgoingAirTemperatureValue"],
                self._data[key]["supplyAirWorkingLevel"],
                self._data[key]["exhaustAirWorkingLevel"],
                self._data[key]["elementInfo"],
                self._data[key]["subWorkingMode"],
                self._data[key]["coolingModeState"],
                self._data[key]["dehumidModeState"],
                self._data[key]["bypassMode"],
                *_other,
            ) = value.split(
                ";",
            )

    def update_resources(self, resources):
        if resources is not None and "vents" in resources:
            vents = resources["vents"]
            for key in vents:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = vents[key]["name"]

    @property
    def vents(self):
        """Returns the vents read from MyGekko"""
        result: list[Vent] = []
        for key, data in self._data.items():
            result.append(Vent(key, data["name"], self))

        return result

    def get_features(self, vent: Vent) -> list[VentFeature]:
        """Returns the supported features of the given vent"""
        result = list()

        if vent and vent.entity_id:
            if vent.entity_id in self._data:
                data = self._data[vent.entity_id]
                if "workingLevel" in data and data["workingLevel"]:
                    result.append(VentFeature.WORKING_LEVEL)
                if "airQualityLevel" in data and data["airQualityLevel"]:
                    result.append(VentFeature.AIR_QUALITY)
                if "co2Value" in data and data["co2Value"]:
                    result.append(VentFeature.CO2)
                if "relativeHumidityLevel" in data and data["relativeHumidityLevel"]:
                    result.append(VentFeature.HUMIDITY)

        return result

    async def set_working_level(
        self, vent: Vent, working_level: VentWorkingLevel
    ) -> None:
        """Sets the working level, OFF is sent as -1"""
        await self._data_provider.write_data(
            vent.resource_path,
            str(working_level if working_level is not VentWorkingLevel.OFF else -1),
        )

    async def set_bypass_state(self, vent: Vent, bypass_state: VentBypassState) -> None:
        """Sets the bypass state"""
        if vent and vent.entity_id:
            await self._data_provider.write_data(
                vent.resource_path, "BY" + str(bypass_state)
            )

    async def set_cooling_mode(self, vent: Vent, cooling_mode: VentCoolingMode) -> None:
        """Sets the cooling mode"""
        if vent and vent.entity_id:
            await self._data_provider.write_data(
                vent.resource_path, "C" + str(cooling_mode)
            )

    async def set_dehumid_mode(self, vent: Vent, dehumid_mode: VentDehumidMode) -> None:
        """Sets the dehumid mode"""
        if vent and vent.entity_id:
            await self._data_provider.write_data(
                vent.resource_path, "D" + str(dehumid_mode)
            )

    async def set_working_mode(
        self,
        vent: Vent,
        working_mode: VentWorkingMode | VentWorkingModeZimmermann,
    ) -> None:
        """Sets the working mode"""
        if vent and vent.entity_id:
            await self._data_provider.write_data(
                vent.resource_path, "M" + str(working_mode)
            )



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.Lights import LightFeature
from PyMyGekko.resources.Lights import LightState


async def var_response(request):
    varResponseFile = open("tests/lights/data/api_var_response_680016.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open("tests/lights/data/api_var_status_response_680016.json")
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_lights(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        lights = api.get_lights()

        assert lights is not None
        assert len(lights) == 5

        assert lights[0].entity_id == "item0"
        assert lights[0].name == "Büro 1 Licht"
        assert lights[0].state == LightState.ON
        assert len(lights[0].supported_features) == 1
        assert LightFeature.ON_OFF in lights[0].supported_features

        assert lights[1].entity_id == "item1"
        assert lights[1].name == "Büro 2 Licht"
        assert lights[1].state == LightState.OFF
        assert len(lights[1].supported_features) == 1
        assert LightFeature.ON_OFF in lights[1].supported_features

        assert lights[2].entity_id == "item2"
        assert lights[2].name == "Büro Küche Licht"
        assert lights[2].state == LightState.OFF
        assert len(lights[2].supported_features) == 1
        assert LightFeature.ON_OFF in lights[1].supported_features

        assert lights[3].entity_id == "group0"
        assert lights[3].name == "ALLE"
        assert lights[3].state is None
        assert lights[3].brightness is None
        assert lights[3].rgb_color is None
        assert len(lights[3].supported_features) == 1
        assert LightFeature.ON_OFF in lights[3].supported_features

        assert lights[4].entity_id == "group1"
        assert lights[4].name == "Licht Aussen"
        assert lights[4].state is None
        assert lights[4].brightness is None
        assert lights[4].rgb_color is None
        assert len(lights[3].supported_features) == 1
        assert LightFeature.ON_OFF in lights[4].supported_features



"""MyGekko AlarmsLogics implementation"""
from __future__ import annotations

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class AlarmsLogic(Entity):
    """Class for MyGekko AlarmsLogic"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: AlarmsLogicValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/alarms_logics/")
        self._value_accessor = value_accessor

    @property
    def value(self) -> float | None:
        """Returns the value"""
        value = self._value_accessor.get_value(self, "currentValue")
        return float(value) if value is not None else None


class AlarmsLogicValueAccessor(EntityValueAccessor):
    """AlarmsLogic value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "alarms_logics" in status:
            alarms_logics = status["alarms_logics"]
            for key in alarms_logics:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if (
                        "sumstate" in alarms_logics[key]
                        and "value" in alarms_logics[key]["sumstate"]
                    ):
                        (
                            self._data[key]["currentValue"],
                            *_other,
                        ) = alarms_logics[
                            key
                        ]["sumstate"]["value"].split(";")

    def update_resources(self, resources):
        if resources is not None and "alarms_logics" in resources:
            alarms_logics = resources["alarms_logics"]
            for key in alarms_logics:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = alarms_logics[key]["name"]

    @property
    def alarms_logics(self):
        """Returns the alarmsLogics read from MyGekko"""
        result: list[AlarmsLogic] = []
        for key, data in self._data.items():
            result.append(AlarmsLogic(key, data["name"], self))

        return result



"""Test for vents in MyGekko software version 680016"""
import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.vents import VentBypassMode
from PyMyGekko.resources.vents import VentBypassState
from PyMyGekko.resources.vents import VentCoolingMode
from PyMyGekko.resources.vents import VentDehumidMode
from PyMyGekko.resources.vents import VentDeviceModel
from PyMyGekko.resources.vents import VentElementInfo
from PyMyGekko.resources.vents import VentFeature
from PyMyGekko.resources.vents import VentSubWorkingMode
from PyMyGekko.resources.vents import VentWorkingLevel
from PyMyGekko.resources.vents import VentWorkingMode


async def var_response(_request):
    """mocked MyGekko resources response"""
    var_response_file = open(
        "tests/vents/data/api_var_response_680016.json", encoding="UTF-8"
    )
    return web.Response(status=200, body=var_response_file.read())


async def var_status_response(_request):
    """mocked MyGekko status response"""
    status_response_file = open(
        "tests/vents/data/api_var_status_response_680016.json", encoding="UTF-8"
    )
    return web.Response(status=200, body=status_response_file.read())


@pytest.fixture
def mock_server(aiohttp_server):
    """mock server"""
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_vents(mock_server):
    """test for the vents"""
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        vents = api.get_vents()

        assert vents is not None
        assert len(vents) == 1

        assert vents[0].entity_id == "item0"
        assert vents[0].name == "Wohnraumlüftung"
        assert vents[0].device_model == VentDeviceModel.PLUGGIT
        assert vents[0].element_info == VentElementInfo.OK
        assert vents[0].air_quality is None
        assert vents[0].co2 == 764.0
        assert vents[0].exhaust_air_temperature == 23.05
        assert vents[0].exhaust_air_working_level == 25.0
        assert vents[0].outgoing_air_temperature == 11.48
        assert vents[0].outside_air_temperature == 8.02
        assert vents[0].relative_humidity == 46.0
        assert vents[0].supply_air_temperature == 21.13
        assert vents[0].supply_air_working_level == 25.0
        assert vents[0].cooling_mode == VentCoolingMode.OFF
        assert vents[0].dehumid_mode == VentDehumidMode.OFF
        assert vents[0].bypass_mode == VentBypassMode.AUTO
        assert vents[0].bypass_state == VentBypassState.AUTO
        assert vents[0].working_level == VentWorkingLevel.LEVEL_1
        assert vents[0].working_mode == VentWorkingMode.MANUAL
        assert vents[0].sub_working_mode == VentSubWorkingMode.EXHAUST_SUPPLY
        assert len(vents[0].supported_features) == 3
        assert VentFeature.CO2 in vents[0].supported_features
        assert VentFeature.HUMIDITY in vents[0].supported_features
        assert VentFeature.WORKING_LEVEL in vents[0].supported_features



"""MyGekko Loads implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class Load(Entity):
    """Class for MyGekko Load"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: LoadValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/loads/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[LoadFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def state(self) -> LoadState | None:
        """Returns the current state"""
        value = self._value_accessor.get_value(self, "currentState")
        return LoadState(int(value)) if value is not None else None

    async def set_state(self, state: LoadState):
        """Sets the state"""
        await self._value_accessor.set_state(self, state)


class LoadState(IntEnum):
    """MyGekko Loads State"""

    OFF = 0
    ON_IMPULSE = 1
    ON_PERMANENT = 2


class LoadFeature(IntEnum):
    """MyGekko Loads Feature"""

    ON_OFF = 0


class LoadValueAccessor(EntityValueAccessor):
    """Loads value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "loads" in status:
            loads = status["loads"]
            for key in loads:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in loads[key] and "value" in loads[key]["sumstate"]:
                        (
                            self._data[key]["currentState"],
                            self._data[key]["elementInfo"],
                            *_other,
                        ) = loads[key]["sumstate"]["value"].split(";")

    def update_resources(self, resources):
        if resources is not None and "loads" in resources:
            loads = resources["loads"]
            for key in loads:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = loads[key]["name"]

    @property
    def loads(self):
        """Returns the loads read from MyGekko"""
        result: list[Load] = []
        for key, data in self._data.items():
            result.append(Load(key, data["name"], self))

        return result

    def get_features(self, load: Load) -> list[LoadFeature]:
        """Returns the supported features"""
        result = list()

        if load and load.entity_id:
            if load.entity_id in self._data:
                data = self._data[load.entity_id]
                if "currentState" in data and data["currentState"]:
                    result.append(LoadFeature.ON_OFF)
        return result

    async def set_state(self, load: Load, state: LoadState) -> None:
        """Sets the state"""
        if load and load.entity_id:
            await self._data_provider.write_data(load.resource_path, str(state))



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase


async def response(request):
    return web.Response(status=200)


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_init():
    async with ClientSession() as session:
        api = MyGekkoApiClientBase("username", "apiKey", "gekkoId", session)

        assert api is not None


@pytest.mark.asyncio
async def test_try_connect(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.try_connect()



"""MyGekko EnergyCosts implementation"""
from __future__ import annotations

import logging
import re

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


_LOGGER: logging.Logger = logging.getLogger(__name__)


class EnergyCost(Entity):
    """Class for MyGekko EnergyCost"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: EnergyCostValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/energycosts/")
        self._value_accessor = value_accessor

    @property
    def sensor_data(self) -> dict[str, any] | None:
        """Returns the sensor data"""
        return self._value_accessor.get_data(self)


class EnergyCostValueAccessor(EntityValueAccessor):
    """EnergyCost value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        self._data_provider.subscribe(self)

    def _transform_value(self, description: str, value: str) -> any:
        m = re.match(r"([^\[]*)\[([^\]]*)\]", description)

        if m is None:
            return None

        typed_value = None

        unit = None if len(m.groups()) == 1 else m.group(2)

        if unit is not None and unit != "Unit" and unit != "DateTime":
            typed_value = float(value)
        else:
            typed_value = value

        return {"name": m.group(1), "unit": unit, "value": typed_value}

    def _decode_values(self, value: str, hardware: str) -> any:
        value_descriptions = []

        if hardware == "legacy":
            value_descriptions = [
                "actPower[kW]",
                "energyToday[kWh]",
                "energyMonth[kWh]",
                "energySum[kWh]",
                "powerMax[kW]",
                "unitEnergy[Unit]",
                "unitPower[Unit]",
                "energyToday6[kWh]",
                "energyToday12[kWh]",
                "energyToday18[kWh]",
                "energyToday24[kWh]",
                "energyYesterd6[kWh]",
                "energyYesterd12[kWh]",
                "energyYesterd18[kWh]",
                "energyYesterd24[kWh]",
                "elementInfo",
                "other",
            ]
        else:
            value_descriptions = [
                "actPower[kW]",
                "energyToday[kWh]",
                "energyMonth[kWh]",
                "energySum[kWh]",
                "powerMax[kW]",
                "unitEnergy[Unit]",
                "unitPower[Unit]",
                "energyToday6[kWh]",
                "energyToday12[kWh]",
                "energyToday18[kWh]",
                "energyToday24[kWh]",
                "energyYesterd6[kWh]",
                "energyYesterd12[kWh]",
                "energyYesterd18[kWh]",
                "energyYesterd24[kWh]",
                "elementInfo",
                "energyYear[kWh]",
                "energyPeriod[kWh]",
                "energyPeriodFrom[DateTime]",
                "counterDirection",
                "other",
            ]
        values = []
        for index, value_parts in enumerate(value.split(";")):
            if index < len(value_descriptions):
                values.append(
                    self._transform_value(value_descriptions[index], value_parts)
                )
            else:
                _LOGGER.error("OutOfBounds access for value %s", value)

        return values

    def update_status(self, status, hardware):
        if status is not None and "energycosts" in status:
            energy_costs = status["energycosts"]

            _LOGGER.debug("EnergyCosts update_status %s", energy_costs)

            for key in energy_costs:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if (
                        "sumstate" in energy_costs[key]
                        and "value" in energy_costs[key]["sumstate"]
                    ):
                        self._data[key]["values"] = self._decode_values(
                            energy_costs[key]["sumstate"]["value"], hardware
                        )

    def update_resources(self, resources):
        if resources is not None and "energycosts" in resources:
            energy_costs = resources["energycosts"]

            _LOGGER.debug("EnergyCosts update_resources %s", energy_costs)

            for key in energy_costs:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = energy_costs[key]["name"]

    @property
    def energy_costs(self):
        """Returns the energyCosts read from MyGekko"""
        result: list[EnergyCost] = []
        for key, data in self._data.items():
            result.append(EnergyCost(key, data["name"], self))

        return result

    def get_data(self, energy_meter: EnergyCost) -> dict[str, any] | None:
        """Returns the data of the given energy meter"""
        if energy_meter and energy_meter.entity_id:
            if (
                energy_meter.entity_id in self._data
                and self._data[energy_meter.entity_id]
            ):
                return self._data[energy_meter.entity_id]
        return None



from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from pytest import fixture
from pytest import mark


async def var_response(request):
    varResponseFile = open("tests/meteo/api_var_response_879015.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open("tests/meteo/api_var_status_response_879015.json")
    return web.Response(status=200, body=statusResponseFile.read())


@fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@mark.asyncio
async def test_get_meteo(mock_server):
    server = await mock_server

    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        meteo = api.get_meteo()

        assert meteo is not None
        assert meteo.sensor_data["twilight"] == "10000.000000"
        assert meteo.sensor_data["brightness"] == "99.000000"
        assert meteo.sensor_data["brightnessw"] == "21.000000"
        assert meteo.sensor_data["brightnesso"] == "16.000000"
        assert meteo.sensor_data["wind"] == "0.003052"
        assert meteo.sensor_data["temperature"] == "23.400000"
        assert meteo.sensor_data["rain"] == "0.000000"



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.Blinds import BlindElementInfo
from PyMyGekko.resources.Blinds import BlindFeature
from PyMyGekko.resources.Blinds import BlindState


async def var_response(request):
    varResponseFile = open("tests/blinds/data/api_var_response_596610.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open("tests/blinds/data/api_var_status_response_596610.json")
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_blinds(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        blinds = api.get_blinds()

        assert blinds is not None
        assert len(blinds) == 13

        assert blinds[0].entity_id == "item0"
        assert blinds[0].name == "Wohnen Terrasse"
        assert blinds[0].position == 100.00
        assert blinds[0].tilt_position is None
        assert blinds[0].state == BlindState.STOP
        assert blinds[0].element_info == BlindElementInfo.OK
        assert len(blinds[0].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[0].supported_features
        assert BlindFeature.SET_POSITION in blinds[0].supported_features

        assert blinds[1].entity_id == "item1"
        assert blinds[1].name == "Wohnen Fenster"
        assert blinds[1].position == 100.00
        assert blinds[1].tilt_position is None
        assert blinds[1].state == BlindState.STOP
        assert blinds[1].element_info == BlindElementInfo.OK
        assert len(blinds[1].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[1].supported_features
        assert BlindFeature.SET_POSITION in blinds[1].supported_features

        assert blinds[2].entity_id == "item2"
        assert blinds[2].name == "Essen"
        assert blinds[2].position == 100.00
        assert blinds[2].tilt_position is None
        assert blinds[2].state == BlindState.STOP
        assert blinds[2].element_info == BlindElementInfo.OK
        assert len(blinds[2].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[2].supported_features
        assert BlindFeature.SET_POSITION in blinds[2].supported_features

        assert blinds[3].entity_id == "item3"
        assert blinds[3].name == "Küche"
        assert blinds[3].position == 100.00
        assert blinds[3].tilt_position is None
        assert blinds[3].state == BlindState.STOP
        assert blinds[3].element_info == BlindElementInfo.OK
        assert len(blinds[3].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[3].supported_features
        assert BlindFeature.SET_POSITION in blinds[3].supported_features

        assert blinds[4].entity_id == "item4"
        assert blinds[4].name == "WC EG"
        assert blinds[4].position == 100.00
        assert blinds[4].tilt_position is None
        assert blinds[4].state == BlindState.STOP
        assert blinds[4].element_info == BlindElementInfo.OK
        assert len(blinds[4].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[4].supported_features
        assert BlindFeature.SET_POSITION in blinds[4].supported_features

        assert blinds[5].entity_id == "item5"
        assert blinds[5].name == "Arbeiten"
        assert blinds[5].position == 100.00
        assert blinds[5].tilt_position is None
        assert blinds[5].state == BlindState.STOP
        assert blinds[5].element_info == BlindElementInfo.OK
        assert len(blinds[5].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[5].supported_features
        assert BlindFeature.SET_POSITION in blinds[5].supported_features

        assert blinds[6].entity_id == "item6"
        assert blinds[6].name == "Eltern"
        assert blinds[6].position == 94.96
        assert blinds[6].tilt_position is None
        assert blinds[6].state == BlindState.DOWN
        assert blinds[6].element_info == BlindElementInfo.OK
        assert len(blinds[6].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[6].supported_features
        assert BlindFeature.SET_POSITION in blinds[6].supported_features

        assert blinds[7].entity_id == "item7"
        assert blinds[7].name == "Kind 1"
        assert blinds[7].position == 100.00
        assert blinds[7].tilt_position is None
        assert blinds[7].state == BlindState.STOP
        assert blinds[7].element_info == BlindElementInfo.OK
        assert len(blinds[7].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[7].supported_features
        assert BlindFeature.SET_POSITION in blinds[7].supported_features

        assert blinds[8].entity_id == "item8"
        assert blinds[8].name == "Kind 2"
        assert blinds[8].position == 100.00
        assert blinds[8].tilt_position is None
        assert blinds[8].state == BlindState.STOP
        assert blinds[8].element_info == BlindElementInfo.OK
        assert len(blinds[8].supported_features) == 2
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[8].supported_features
        assert BlindFeature.SET_POSITION in blinds[8].supported_features

        assert blinds[9].entity_id == "item9"
        assert blinds[9].name == "Bad OG"
        assert blinds[9].position == 100.00
        assert blinds[9].tilt_position == 50.00
        assert blinds[9].state == BlindState.STOP
        assert blinds[9].element_info == BlindElementInfo.OK
        assert len(blinds[9].supported_features) == 3
        assert BlindFeature.OPEN_CLOSE_STOP in blinds[9].supported_features
        assert BlindFeature.SET_POSITION in blinds[9].supported_features
        assert BlindFeature.SET_TILT_POSITION in blinds[9].supported_features

        assert blinds[10].entity_id == "group0"
        assert blinds[10].name == ""
        assert blinds[10].position is None
        assert blinds[10].tilt_position is None
        assert blinds[10].state == BlindState.STOP
        assert len(blinds[10].supported_features) == 1
        assert BlindFeature.OPEN_CLOSE in blinds[10].supported_features

        assert blinds[11].entity_id == "group1"
        assert blinds[11].name == "EG"
        assert blinds[11].position is None
        assert blinds[11].tilt_position is None
        assert blinds[11].state == BlindState.STOP
        assert len(blinds[11].supported_features) == 1
        assert BlindFeature.OPEN_CLOSE in blinds[11].supported_features



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.Loads import LoadFeature
from PyMyGekko.resources.Loads import LoadState


async def var_response(request):
    varResponseFile = open("tests/loads/data/api_var_response_680016.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open("tests/loads/data/api_var_status_response_680016.json")
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_loads(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        loads = api.get_loads()

        assert loads is not None
        assert len(loads) == 3

        assert loads[0].entity_id == "item0"
        assert loads[0].name == "Balkon Steckdose"
        assert loads[0].state == LoadState.ON_PERMANENT
        assert len(loads[0].supported_features) == 1
        assert LoadFeature.ON_OFF in loads[0].supported_features

        assert loads[1].entity_id == "item1"
        assert loads[1].name == "Terrasse Steckdose"
        assert loads[1].state == LoadState.OFF
        assert len(loads[1].supported_features) == 1
        assert LoadFeature.ON_OFF in loads[1].supported_features

        assert loads[2].entity_id == "item2"
        assert loads[2].name == "HWR Steckdose"
        assert loads[2].state == LoadState.OFF
        assert len(loads[2].supported_features) == 1
        assert LoadFeature.ON_OFF in loads[2].supported_features



"""MyGekko Hotwater_systems implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class HotWaterSystem(Entity):
    """Class for MyGekko HotWaterSystem"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: HotWaterSystemValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/hotwater_systems/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[HotWaterSystemFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def state(self) -> HotWaterSystemState | None:
        """Return the current state"""
        value = self._value_accessor.get_value(self, "state")
        return HotWaterSystemState(int(value)) if value is not None else None

    async def set_state(self, state: HotWaterSystemState):
        """Sets the state"""
        await self._value_accessor.set_state(self, state)

    @property
    def target_temperature(self) -> float | None:
        """Return the current target temperature"""
        value = self._value_accessor.get_value(self, "setpointTemp")
        return float(value) if value is not None else None

    async def set_target_temperature(self, target_temperature: float):
        """Sets the target temperature"""
        await self._value_accessor.set_target_temperature(self, target_temperature)

    @property
    def current_temperature_top(self) -> float | None:
        """Return the current top temperature"""
        value = self._value_accessor.get_value(self, "topTemp")
        return float(value) if value is not None else None

    @property
    def current_temperature_bottom(self) -> float | None:
        """Return the current bottom temperature"""
        value = self._value_accessor.get_value(self, "bottomTemp")
        return float(value) if value is not None else None


class HotWaterSystemState(IntEnum):
    """MyGekko HotWaterSystem state"""

    OFF = 0
    ON = 1


class HotWaterSystemFeature(IntEnum):
    """MyGekko HotWaterSystem Feature"""

    ON_OFF = 0
    TARGET_TEMPERATURE = 1
    BOTTOM_TEMPERATURE = 2
    TOP_TEMPERATURE = 3


class HotWaterSystemValueAccessor(EntityValueAccessor):
    """HotWaterSystem value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "hotwater_systems" in status:
            hotwater_systems = status["hotwater_systems"]
            for key in hotwater_systems:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if (
                        "sumstate" in hotwater_systems[key]
                        and "value" in hotwater_systems[key]["sumstate"]
                    ):
                        (
                            self._data[key]["type"],
                            self._data[key]["cooling"],
                            self._data[key]["setpointTemp"],
                            self._data[key]["topTemp"],
                            self._data[key]["bottomTemp"],
                            self._data[key]["collectorTemp"],
                            self._data[key]["state"],
                            self._data[key]["sum"],
                            *_other,
                        ) = hotwater_systems[key]["sumstate"]["value"].split(";")

    def update_resources(self, resources):
        if resources is not None and "hotwater_systems" in resources:
            hotwater_systems = resources["hotwater_systems"]
            for key in hotwater_systems:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = hotwater_systems[key]["name"]

    @property
    def hotwater_systems(self):
        """Returns the hotwater_systems read from MyGekko"""
        result: list[HotWaterSystem] = []
        for key, data in self._data.items():
            result.append(HotWaterSystem(key, data["name"], self))

        return result

    def get_features(
        self, hotwater_system: HotWaterSystem
    ) -> list[HotWaterSystemFeature]:
        """Returns the supported features"""
        result = list()

        if hotwater_system and hotwater_system.entity_id:
            if hotwater_system.entity_id in self._data:
                data = self._data[hotwater_system.entity_id]
                if "state" in data and data["state"]:
                    result.append(HotWaterSystemFeature.ON_OFF)
                if "setpointTemp" in data and data["setpointTemp"]:
                    result.append(HotWaterSystemFeature.TARGET_TEMPERATURE)
                if "bottomTemp" in data and data["bottomTemp"]:
                    result.append(HotWaterSystemFeature.BOTTOM_TEMPERATURE)
                if "topTemp" in data and data["topTemp"]:
                    result.append(HotWaterSystemFeature.TOP_TEMPERATURE)

        return result

    async def set_state(
        self, hotwater_system: HotWaterSystem, state: HotWaterSystemState
    ) -> None:
        """Sets the state"""
        if hotwater_system and hotwater_system.entity_id:
            await self._data_provider.write_data(
                hotwater_system.resource_path, str(state)
            )

    async def set_target_temperature(
        self, hotwater_system: HotWaterSystem, target_temperature: float
    ) -> None:
        """Sets the target temperature"""
        if hotwater_system and hotwater_system.entity_id:
            await self._data_provider.write_data(
                hotwater_system.resource_path, "T" + str(target_temperature)
            )



"""MyGekko Meteos implementation"""
from __future__ import annotations

import logging
from typing import Any

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import ReadOnlyEntity


_LOGGER: logging.Logger = logging.getLogger(__name__)


class Meteo(ReadOnlyEntity):
    """Class for MyGekko Meteo"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: MeteoValueAccessor
    ) -> None:
        super().__init__(entity_id, name)
        self._value_accessor = value_accessor

    @property
    def sensor_data(self) -> dict[str, Any]:
        """Returns the sensor data"""
        return self._value_accessor.get_data(self)


class MeteoValueAccessor(EntityValueAccessor):
    """Meteo value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        self._data_provider.subscribe(self)

    def update_status(self, status, hardware):
        meteo = status.get("globals", {}).get("meteo", {})
        for key, value in meteo.items():
            if "value" in value:
                self._data[key] = value["value"]
            else:
                self._data.setdefault(key, {})

    def update_resources(self, resources):
        """Nothing to do here since Meteo data is in status and no additional resources are available"""

    @property
    def meteo(self):
        """Returns the meteo read from MyGekko"""
        return Meteo("meteo", "Weather", self)

    def get_data(self, meteo: Meteo) -> dict[str, Any]:
        """Returns the data of the given meteo"""
        return self._data



"""MyGekko Actions implementation"""
from __future__ import annotations

from enum import IntEnum

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class Action(Entity):
    """Class for MyGekko Action"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: ActionValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/actions/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[ActionFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def state(self) -> ActionState | None:
        """Returns the current action state"""
        value = self._value_accessor.get_value(self, "currentState")
        return ActionState(int(value)) if value is not None else None

    async def set_state(self, state: ActionState):
        """Sets the action state"""
        await self._value_accessor.set_state(self, state)

    @property
    def start_condition_state(self) -> ActionState | None:
        """Returns the start condition state"""
        value = self._value_accessor.get_value(self, "startConditionState")
        return ActionState(int(value)) if value is not None else None


class ActionState(IntEnum):
    """MyGekko Action State"""

    OFF = 0
    ON = 1


class ActionFeature(IntEnum):
    """MyGekko Action Features"""

    ON_OFF = 0


class ActionValueAccessor(EntityValueAccessor):
    """Action value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "actions" in status:
            actions = status["actions"]
            for key in actions:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if (
                        "sumstate" in actions[key]
                        and "value" in actions[key]["sumstate"]
                    ):
                        (
                            self._data[key]["currentState"],
                            self._data[key]["startConditionState"],
                            self._data[key]["elementInfo"],
                            *_other,
                        ) = actions[key]["sumstate"]["value"].split(";")

    def update_resources(self, resources):
        if resources is not None and "actions" in resources:
            actions = resources["actions"]
            for key in actions:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = actions[key]["name"]

    @property
    def actions(self):
        """Returns the actions read from MyGekko"""
        result: list[Action] = []
        for key, data in self._data.items():
            result.append(Action(key, data["name"], self))

        return result

    def get_features(self, action: Action) -> list[ActionFeature]:
        """Returns the supported features"""
        result = list()

        if action and action.entity_id:
            if action.entity_id in self._data:
                data = self._data[action.entity_id]
                if "currentState" in data and data["currentState"]:
                    result.append(ActionFeature.ON_OFF)

        return result

    async def set_state(self, action: Action, state: ActionState) -> None:
        """Sets the state"""
        if action and action.entity_id:
            await self._data_provider.write_data(action.resource_path, str(state))



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.RoomTemps import RoomTempsFeature
from PyMyGekko.resources.RoomTemps import RoomTempsMode


async def var_response(request):
    varResponseFile = open("tests/room_temps/data/api_var_response_slide.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open(
        "tests/room_temps/data/api_var_status_response_slide.json"
    )
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_room_temps(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        roomTemps = api.get_room_temps()

        assert roomTemps is not None
        assert len(roomTemps) == 7

        assert roomTemps[0].entity_id == "item0"
        assert roomTemps[0].name == "Wohnen"
        assert roomTemps[0].current_temperature == 23.94000053405762
        assert roomTemps[0].target_temperature == 22.00
        assert roomTemps[0].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[0].humidity is None
        assert len(roomTemps[0].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[0].supported_features

        assert roomTemps[1].entity_id == "item1"
        assert roomTemps[1].name == "Gäste-Wc"
        assert roomTemps[1].current_temperature == 22.5
        assert roomTemps[1].target_temperature == 22.00
        assert roomTemps[1].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[1].humidity is None
        assert len(roomTemps[1].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[1].supported_features

        assert roomTemps[2].entity_id == "item2"
        assert roomTemps[2].name == "Kind 1"
        assert roomTemps[2].current_temperature == 23.61999893188477
        assert roomTemps[2].target_temperature == 19.00
        assert roomTemps[2].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[2].humidity is None
        assert len(roomTemps[2].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[2].supported_features

        assert roomTemps[3].entity_id == "item3"
        assert roomTemps[3].name == "Schlafzimmer"
        assert roomTemps[3].current_temperature == 21.95000076293945
        assert roomTemps[3].target_temperature == 22.00
        assert roomTemps[3].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[3].humidity is None
        assert len(roomTemps[3].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[3].supported_features

        assert roomTemps[4].entity_id == "item4"
        assert roomTemps[4].name == "Kind 2"
        assert roomTemps[4].current_temperature == 24.97999954223633
        assert roomTemps[4].target_temperature == 23.00
        assert roomTemps[4].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[4].humidity is None
        assert len(roomTemps[4].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[4].supported_features

        assert roomTemps[5].entity_id == "item5"
        assert roomTemps[5].name == "Kind 3"
        assert roomTemps[5].current_temperature == 22.81599998474121
        assert roomTemps[5].target_temperature == 21.00
        assert roomTemps[5].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[5].humidity is None
        assert len(roomTemps[5].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[5].supported_features

        assert roomTemps[6].entity_id == "item6"
        assert roomTemps[6].name == "Bad"
        assert roomTemps[6].current_temperature == 24.07999992370605
        assert roomTemps[6].target_temperature == 22.0
        assert roomTemps[6].working_mode == RoomTempsMode.REDUCED
        assert roomTemps[6].humidity is None
        assert len(roomTemps[6].supported_features) == 1
        assert RoomTempsFeature.TARGET_TEMPERATURE in roomTemps[6].supported_features



"""MyGekko Lights implementation"""
from __future__ import annotations

from enum import IntEnum
from math import ceil

from PyMyGekko.data_provider import DataProviderBase
from PyMyGekko.data_provider import EntityValueAccessor
from PyMyGekko.resources import Entity


class Light(Entity):
    """Class for MyGekko Light"""

    def __init__(
        self, entity_id: str, name: str, value_accessor: LightValueAccessor
    ) -> None:
        super().__init__(entity_id, name, "/lights/")
        self._value_accessor = value_accessor
        self._supported_features = self._value_accessor.get_features(self)

    @property
    def supported_features(self) -> list[LightFeature]:
        """Returns the supported features"""
        return self._supported_features

    @property
    def state(self) -> LightState | None:
        """Returns the current state"""
        value = self._value_accessor.get_value(self, "currentState")
        return LightState(int(value)) if value is not None else None

    async def set_state(self, state: LightState):
        """Sets the state"""
        await self._value_accessor.set_state(self, state)

    @property
    def brightness(self) -> int | None:
        """Returns the current brightness"""
        value = self._value_accessor.get_value(self, "dimLevel")
        return ceil(float(value)) if value is not None else None

    async def set_brightness(self, brightness: int):
        """Sets the brightness"""
        await self._value_accessor.set_brightness(self, brightness)

    @property
    def rgb_color(self) -> tuple[int, int, int] | None:
        """Returns the current rgb color"""
        value = self._value_accessor.get_value(self, "rgbColor")
        return ColorUtilities.decimal_to_rgb(int(value)) if value is not None else None

    async def set_rgb_color(self, rgb_color: tuple[int, int, int]):
        """Sets the rgb value"""
        await self._value_accessor.set_rgb_color(self, rgb_color)


class LightState(IntEnum):
    """MyGekko Lights State"""

    OFF = 0
    ON = 1


class LightFeature(IntEnum):
    """MyGekko Lights Feature"""

    ON_OFF = 0
    DIMMABLE = 1
    RGB_COLOR = 2


class LightValueAccessor(EntityValueAccessor):
    """Lights value accessor"""

    def __init__(self, data_provider: DataProviderBase):
        self._data = {}
        self._data_provider = data_provider
        data_provider.subscribe(self)

    def update_status(self, status, hardware):
        if status is not None and "lights" in status:
            lights = status["lights"]
            for key in lights:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in lights[key] and "value" in lights[key]["sumstate"]:
                        (
                            self._data[key]["currentState"],
                            self._data[key]["dimLevel"],
                            self._data[key]["rgbColor"],
                            self._data[key]["tunableWhiteLevel"],
                            self._data[key]["elementInfo"],
                            *_other,
                        ) = lights[key]["sumstate"]["value"].split(";")

                if key.startswith("group"):
                    if key not in self._data:
                        self._data[key] = {}

                    if "sumstate" in lights[key] and "value" in lights[key]["sumstate"]:
                        (
                            self._data[key]["currentState"],
                            *_other,
                        ) = lights[key][
                            "sumstate"
                        ]["value"].split(
                            ";",
                        )

    def update_resources(self, resources):
        if resources is not None and "lights" in resources:
            lights = resources["lights"]
            for key in lights:
                if key.startswith("item"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = lights[key]["name"]

                if key.startswith("group"):
                    if key not in self._data:
                        self._data[key] = {}
                    self._data[key]["name"] = lights[key]["name"]

    @property
    def lights(self):
        """Returns the lights read from MyGekko"""
        result: list[Light] = []
        for key, data in self._data.items():
            result.append(Light(key, data["name"], self))

        return result

    def get_features(self, light: Light) -> list[LightFeature]:
        """Returns the supported features"""
        result = list()

        if light and light.entity_id:
            if light.entity_id in self._data:
                data = self._data[light.entity_id]

                if light.entity_id.startswith("group"):
                    # on/off feature is the only feature for groups
                    result.append(LightFeature.ON_OFF)
                    return result

                if "currentState" in data and data["currentState"]:
                    result.append(LightFeature.ON_OFF)

                if "dimLevel" in data and data["dimLevel"]:
                    result.append(LightFeature.DIMMABLE)

                if "rgbColor" in data and data["rgbColor"]:
                    result.append(LightFeature.RGB_COLOR)

        return result

    async def set_state(self, light: Light, state: LightState) -> None:
        """Sets the state"""
        if light and light.entity_id:
            await self._data_provider.write_data(light.resource_path, str(state))

    async def set_brightness(self, light: Light, brightness: int) -> None:
        """Sets the brightness"""
        if light and light.entity_id and brightness >= 0 and brightness <= 100:
            await self._data_provider.write_data(
                light.resource_path, "D" + str(brightness)
            )

    async def set_rgb_color(
        self, light: Light, rgb_color: tuple[int, int, int]
    ) -> None:
        """Sets the rgb value"""
        if (
            light
            and light.entity_id
            and rgb_color[0] >= 0
            and rgb_color[0] <= 255
            and rgb_color[1] >= 0
            and rgb_color[1] <= 255
            and rgb_color[2] >= 0
            and rgb_color[2] <= 255
        ):
            decimal_rbg_color = (
                (rgb_color[0] << 16) + (rgb_color[1] << 8) + rgb_color[2]
            )
            await self._data_provider.write_data(
                light.resource_path, "C" + str(decimal_rbg_color)
            )


class ColorUtilities:
    """Color Utility class"""

    @staticmethod
    def decimal_to_rgb(decimal_rgb_color: int) -> tuple[int, int, int]:
        """Converts a decimal color representation to a rgb tuple"""
        return (
            (decimal_rgb_color >> 16) & 255,
            (decimal_rgb_color >> 8) & 255,
            decimal_rgb_color & 255,
        )



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase


async def var_response(request):
    varResponseFile = open("tests/energy_costs/data/api_var_response_643612.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open(
        "tests/energy_costs/data/api_var_status_response_643612.json"
    )
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_energy_costs(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        energy_costs = api.get_energy_costs()

        assert energy_costs is not None
        assert len(energy_costs) == 4

        assert energy_costs[0].entity_id == "item0"
        assert energy_costs[0].name == "Meter 1"
        assert len(energy_costs[0].sensor_data) == 2
        assert len(energy_costs[0].sensor_data["values"]) == 20
        assert energy_costs[0].sensor_data["name"] == "Meter 1"

        assert energy_costs[0].sensor_data["values"][0]["name"] == "actPower"
        assert energy_costs[0].sensor_data["values"][0]["unit"] == "kW"
        assert energy_costs[0].sensor_data["values"][0]["value"] == 0.94

        assert energy_costs[0].sensor_data["values"][1]["name"] == "energyToday"
        assert energy_costs[0].sensor_data["values"][1]["unit"] == "kWh"
        assert energy_costs[0].sensor_data["values"][1]["value"] == 0.73

        assert energy_costs[0].sensor_data["values"][2]["name"] == "energyMonth"
        assert energy_costs[0].sensor_data["values"][2]["unit"] == "kWh"
        assert energy_costs[0].sensor_data["values"][2]["value"] == 108.97

        assert energy_costs[1].entity_id == "item1"
        assert energy_costs[1].name == "Meter 2"
        assert len(energy_costs[1].sensor_data) == 2
        assert len(energy_costs[1].sensor_data["values"]) == 20
        assert energy_costs[1].sensor_data["name"] == "Meter 2"

        assert energy_costs[1].sensor_data["values"][0]["name"] == "actPower"
        assert energy_costs[1].sensor_data["values"][0]["unit"] == "kW"
        assert energy_costs[1].sensor_data["values"][0]["value"] == -1.71

        assert energy_costs[1].sensor_data["values"][1]["name"] == "energyToday"
        assert energy_costs[1].sensor_data["values"][1]["unit"] == "kWh"
        assert energy_costs[1].sensor_data["values"][1]["value"] == 2.52

        assert energy_costs[1].sensor_data["values"][2]["name"] == "energyMonth"
        assert energy_costs[1].sensor_data["values"][2]["unit"] == "kWh"
        assert energy_costs[1].sensor_data["values"][2]["value"] == 259.4

        assert energy_costs[2].entity_id == "item2"
        assert energy_costs[2].name == "Photovoltaics"
        assert len(energy_costs[2].sensor_data) == 2
        assert len(energy_costs[2].sensor_data["values"]) == 20
        assert energy_costs[2].sensor_data["name"] == "Photovoltaics"

        assert energy_costs[2].sensor_data["values"][0]["name"] == "actPower"
        assert energy_costs[2].sensor_data["values"][0]["unit"] == "kW"
        assert energy_costs[2].sensor_data["values"][0]["value"] == -3.03

        assert energy_costs[2].sensor_data["values"][1]["name"] == "energyToday"
        assert energy_costs[2].sensor_data["values"][1]["unit"] == "kWh"
        assert energy_costs[2].sensor_data["values"][1]["value"] == 0.07

        assert energy_costs[2].sensor_data["values"][2]["name"] == "energyMonth"
        assert energy_costs[2].sensor_data["values"][2]["unit"] == "kWh"
        assert energy_costs[2].sensor_data["values"][2]["value"] == 3.0

        assert energy_costs[3].entity_id == "item3"
        assert energy_costs[3].name == "Swimming Pool"
        assert len(energy_costs[3].sensor_data) == 2
        assert len(energy_costs[3].sensor_data["values"]) == 20
        assert energy_costs[3].sensor_data["name"] == "Swimming Pool"

        assert energy_costs[3].sensor_data["values"][0]["name"] == "actPower"
        assert energy_costs[3].sensor_data["values"][0]["unit"] == "kW"
        assert energy_costs[3].sensor_data["values"][0]["value"] == 0.01

        assert energy_costs[3].sensor_data["values"][1]["name"] == "energyToday"
        assert energy_costs[3].sensor_data["values"][1]["unit"] == "kWh"
        assert energy_costs[3].sensor_data["values"][1]["value"] == 0.08

        assert energy_costs[3].sensor_data["values"][2]["name"] == "energyMonth"
        assert energy_costs[3].sensor_data["values"][2]["unit"] == "kWh"
        assert energy_costs[3].sensor_data["values"][2]["value"] == 60.92



import pytest
from aiohttp import ClientSession
from aiohttp import web
from PyMyGekko import MyGekkoApiClientBase
from PyMyGekko.resources.HotWaterSystems import HotWaterSystemFeature
from PyMyGekko.resources.HotWaterSystems import HotWaterSystemState


async def var_response(request):
    varResponseFile = open("tests/hotwater_systems/data/api_var_response_596610.json")
    return web.Response(status=200, body=varResponseFile.read())


async def var_status_response(request):
    statusResponseFile = open(
        "tests/hotwater_systems/data/api_var_status_response_596610.json"
    )
    return web.Response(status=200, body=statusResponseFile.read())


@pytest.fixture
def mock_server(aiohttp_server):
    app = web.Application()
    app.router.add_get("/api/v1/var", var_response)
    app.router.add_get("/api/v1/var/status", var_status_response)
    return aiohttp_server(app)


@pytest.mark.asyncio
async def test_get_hotwater_systems(mock_server):
    server = await mock_server
    async with ClientSession() as session:
        api = MyGekkoApiClientBase(
            {},
            session,
            scheme=server.scheme,
            host=server.host,
            port=server.port,
        )

        await api.read_data()
        hotWaterSystems = api.get_hot_water_systems()

        assert hotWaterSystems is not None
        assert len(hotWaterSystems) == 1

        assert hotWaterSystems[0].entity_id == "item0"
        assert hotWaterSystems[0].name == "WW Boiler"
        assert hotWaterSystems[0].state == HotWaterSystemState.ON
        assert hotWaterSystems[0].target_temperature == 48.0
        assert hotWaterSystems[0].current_temperature_bottom == 37.7
        assert hotWaterSystems[0].current_temperature_top == 43.8
        assert len(hotWaterSystems[0].supported_features) == 4
        assert HotWaterSystemFeature.ON_OFF in hotWaterSystems[0].supported_features
        assert (
            HotWaterSystemFeature.TARGET_TEMPERATURE
            in hotWaterSystems[0].supported_features
        )
        assert (
            HotWaterSystemFeature.TOP_TEMPERATURE
            in hotWaterSystems[0].supported_features
        )
        assert (
            HotWaterSystemFeature.BOTTOM_TEMPERATURE
            in hotWaterSystems[0].supported_features
        )
